// This file is part of VKF, a CPython library
// for Machine Learning based on Lattice Theory.
//
// This Source Code Form is subject to the terms of the X11 (MIT) License.
// If a copy of the X11 License was not distributed with this file, 
// You can obtain one at https://opensource.org/licenses/MIT/.
//
// X11 (MIT) License
// 
// Copyright (c) 2020 Dmitry V. Vinogradov
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef DOTPICTURE_HPP
#define DOTPICTURE_HPP

#include <algorithm> // for std::for_each
#include <cassert> // for assert
#include <iostream> // for std::cout,std::cerr
#include <string> // for std::string

class DotPicture : public std::vector<std::string> {
    private:
        const std::string attribute_name;
        std::string dot_string;
    public:
        /** default constructor */
        DotPicture(const std::string& _name) : attribute_name(_name) { }
        /** Access to the name of encoded attribute
         * \return attribute_name */
        const std::string GetAttributeName() const { return attribute_name; }
        std::string ClearDotString() { dot_string.clear(); return dot_string; }
        std::string AppendString(const std::string &substring) { dot_string.append(substring); return dot_string; }
        std::string ShowDotString() const { return dot_string; }
};

#endif // DOTPICTURE_HPP
